@@ -2,7 +2,7 @@
 package MRTG_lib;
 
 ###################################################################
-# MRTG 2.9.12a  Support library MRTG_lib.pm
+# MRTG 2.9.17  Support library MRTG_lib.pm
 ###################################################################
 # Created by Tobias Oetiker <oetiker@ee.ethz.ch>
 #            and Dave Rand <dlr@bungi.com>
@@ -15,9 +15,9 @@ package MRTG_lib;
 #
 ###################################################################
 
-require 5.003;
+require 5.005;
 use strict;
-use SNMP_util "0.77";
+use SNMP_util "0.86";
 use vars qw($OS $SL $PS @EXPORT @ISA $VERSION %mrtgrules);
 
 BEGIN {
@@ -39,12 +39,12 @@ BEGIN {
 
 require Exporter;
 @ISA = qw(Exporter $VERSION);
-@EXPORT = qw(readcfg cfgcheck
-	     datestr expistr ensureSL
+@EXPORT = qw(readcfg cfgcheck setup_loghandlers
+	     datestr expistr ensureSL timestamp
              demonize_me debug log2rrd storeincache
 	     populateconfcache readconfcache writeconfcache);
 
-$VERSION = 2.090011;
+$VERSION = 2.090017;
 
 
 %mrtgrules =
@@ -916,14 +916,14 @@ sub demonize_me ($) {
     my $pidfile = shift;
     print "Daemonizing MRTG ...\n";
     if ( $OS eq 'NT' ) {
-            require Win32::Console;
-            my $CONSOLE = new Win32::Console;
-            #detach process from Console
-            $CONSOLE->Flush();
-            $CONSOLE->Free();
-            #Redirect errors and messages to log files
-            open STDOUT,'>mtrgout.log';
-            open STDERR,'>mtrgerr.log';
+	print "Do Not close this window. Or MRTG will die\n";
+#            require Win32::Console;
+#            my $CONSOLE = new Win32::Console;
+	#    detach process from Console
+#            $CONSOLE->Flush();
+#            $CONSOLE->Free();
+#	    $CONSOLE->Alloc();
+#	    $CONSOLE->Mode()
      } else {
            defined (my $pid = fork) or die "Can't fork: $!";
            if ($pid) {
@@ -960,9 +960,9 @@ sub targparser ($$) {
                  '   \.   (\d+)|'.              # interface number
 	         '   /    (\d+(?:\.\d+)+)|'.    # ip interface sclector
 	         '   !    ([0-9a-f]+(?:-[0-9a-f]+)+)|'. # ethernet interface
-	         '   \\\\ ((?:\\\\[:& ]|[^\s:])+)|'. # if description (perm \: \& \ )
+	         '   \\\\ ((?:\\\\[:& ]|[^\s:&])+)|'. # if description (perm \: \& \ )
 	         '   %    (\d+)|'.                # if Type
-	         '   \#   ((?:\\\\[:& ]|[^\s:])+)'. # if name (perm \: \& \ )
+	         '   \#   ((?:\\\\[:& ]|[^\s:&])+)'. # if name (perm \: \& \ )
 	         ')';
     # programm targets
     while (
@@ -1070,9 +1070,9 @@ sub targparser ($$) {
                       (?:   (\d+)|       # interface number
 	                 /  (\d+(?:\.\d+)+)|  # ip interface sclector
 	                 !  ([0-9a-f]+(?:-[0-9a-f]+)+)|   # ethernet interface selector
-	                 \\ ((?:\\[&: ]|[^\s:])+)| # description if selector ('\:' and '\ ' allowed)
+	                 \\ ((?:\\[&: ]|[^\s:&])+)| # description if selector ('\:' and '\ ' allowed)
            	         %  (\d+)|                 # if Type
-	                 \# ((?:\\[&: ]|[^\s:])+)  # name if selector ('\:' and '\ ' allowed)
+	                 \# ((?:\\[&: ]|[^\s:&])+)  # name if selector ('\:' and '\ ' allowed)
                        )
 	               :               # separator
 	               ((?:\\[@ ]|[^\s@])+) # community string ('\@' and '\ ' allowed)
@@ -1198,11 +1198,11 @@ sub storeincache ($$$$$){
     if (defined $$confcache{$host}{$method}{$key} and 
 	$$confcache{$host}{$method}{$key} ne $value) {
         $$confcache{$host}{$method}{$key} = "Dup";
+	debug('snpo',"confcache $host $method $key --> $value (duplicate)");
     } else {
         $$confcache{$host}{$method}{$key} = $value;
-        
+	debug('snpo',"confcache $host $method $key --> $value");
     }
-    debug('snpo',"confcache $host $method $key --> $value");
 
 }
 
@@ -1211,51 +1211,60 @@ sub populateconfcache ($$$$) {
     my $host = shift;
     my $reread = shift;
     my $snmpoptions = shift || {};
+    my $snmp_errlevel = $SNMP_Session::suppress_warnings;
+    $SNMP_Session::suppress_warnings = 3;    
 
     return if defined $$confcache{$host} and not $reread;
     # clear confcache for host;
-    delete $$confcache{$host};
-    my @ret = snmpwalk($host, $snmpoptions, "ifDescr");
-    foreach my $ret (@ret)
-      {
-	  my ($oid, $desc) = split(':', $ret, 2);
-	  storeincache($confcache,$host,"Descr",$desc,$oid);
-      };
-    @ret = snmpwalk($host, $snmpoptions, "ifName");
-    foreach my $ret (@ret)
-      {
-	  my ($oid, $desc) = split(':', $ret, 2);
-	  storeincache($confcache,$host,"Name",$desc,$oid);
-      };
-    @ret = snmpwalk($host, $snmpoptions, "ifType");
-    foreach my $ret (@ret)
-      {
-	  my ($oid, $desc) = split(':', $ret, 2);
-	  storeincache($confcache,$host,"Type",$desc,$oid);
-      };
-    @ret = snmpwalk($host, $snmpoptions, "ipAdEntIfIndex");
-    foreach my $ret (@ret)
-      {
-	  my ($addr, $oid) = split(':', $ret, 2);
-	  storeincache($confcache,$host,"Ip",$addr,$oid);
-      };
 
+    delete $$confcache{$host};
+    my @ret;
+    my %tables = ( ifDescr => 'Descr',
+		   ifName  => 'Name',
+		   ifType  => 'Type',
+		   ipAdEntIfIndex => 'Ip' );
+
+    foreach my $node ( keys %tables ) {
+	$SNMP_Session::errmsg = undef;
+	@ret = snmpwalk($host, $snmpoptions, $node);
+	unless ( $SNMP_Session::errmsg){
+	    foreach my $ret (@ret)
+	      {
+		  my ($oid, $desc) = split(':', $ret, 2);
+		  if ($tables{$node} eq 'Ip') {
+		      storeincache($confcache,$host,$tables{$node},$oid,$desc);
+		  } else {
+		      storeincache($confcache,$host,$tables{$node},$desc,$oid);
+		  }
+	      };
+	} else {
+	    debug('snpo',"Skipping $node scanning because $host does not seem to support it");
+	}
+    }
+    $SNMP_Session::errmsg = undef;
     @ret = snmpwalk($host, $snmpoptions, "ifPhysAddress");
-    foreach my $ret (@ret)
-      {
-	  my ($oid, $bin) = split(':', $ret, 2);
-	  my $eth = unpack 'H*', $bin;
-	  my @eth;
-	  while ($eth =~ s/^..//){
-	      push @eth, $&;
-	  }
-	  my $phys=join '-', @eth;
-	  storeincache($confcache,$host,"Eth",$phys,$oid);
-      }
+    unless ( $SNMP_Session::errmsg){
+	foreach my $ret (@ret)
+	  {
+	      my ($oid, $bin) = split(':', $ret, 2);
+	      my $eth = unpack 'H*', $bin; 
+ 	      my @eth;
+	      while ($eth =~ s/^..//){
+	        push @eth, $&;
+	      }
+	      my $phys=join '-', @eth;
+	      storeincache($confcache,$host,"Eth",$phys,$oid);
+           }
+     } else {
+            debug('snpo',"Skipping ifPhysAddress scanning because $host does not seem to support it");
+     }
+
      if (ref $$confcache{___updated} ne 'ARRAY') {
         $$confcache{___updated} = []; #init to empty array
      }
      push @{$$confcache{___updated}}, $host;
+
+    $SNMP_Session::suppress_warnings = $snmp_errlevel;    
 }
 
 sub log2rrd ($$$) {
@@ -1470,11 +1479,80 @@ sub debug ($$) {
     return unless scalar @main::DEBUG;
     my $tag = shift;
     my $msg = shift;
-    print STDERR "--",$tag,": ",$msg,"\n"
-      if grep {$_ eq $tag} @main::DEBUG;
+    return unless grep {$_ eq $tag} @main::DEBUG;
+    warn "--",$tag,": ",$msg,"\n";
     return;
 }
 
+# timestamp
+sub timestamp () {
+    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) =
+                                                localtime(time);
+    $year += 1900;
+    $mon += 1;
+    return sprintf "%4d-%02d-%02d %02d:%02d:%02d", $year,$mon,$mday,$hour,$min,$sec;
+}
+
+# configure __DIE__ and __WARN__
+       
+sub setup_loghandlers ($){
+    $::global_logfile = $_[0];
+    for($_[0]){
+	/^eventlog$/i && do {
+	    require Win32::EventLog;
+	    $SIG{__WARN__} = sub {
+		my $EventLog = Win32::EventLog->new('MRTG');
+		my $Type = ($_[0] =~ /warning/) ? 
+		  &Win32::EventLog::EVENTLOG_WARNING_TYPE : 
+		  &Win32::EventLog::EVENTLOG_INFORMATION_TYPE;
+		my $Msg = $_[0];
+		$Msg =~ s/\n/\r\n/g;
+                $Msg =~ s/[\n\r]$//g;
+		$EventLog->Report({
+		    EventID => 1000,
+		      EventType => $Type,
+		      Strings => $Msg });
+		$EventLog->Close;
+	    };
+	    $SIG{__DIE__} = sub {
+		my $EventLog = Win32::EventLog->new('MRTG');
+		my $Msg = $_[0];
+		$Msg =~ s/\n/\r\n/g;
+                $Msg =~ s/[\n\r]$//g;
+		$EventLog->Report({
+		    EventID => 1000,
+		      EventType => &Win32::EventLog::EVENTLOG_ERROR_TYPE,
+		      Strings => $Msg });
+		$EventLog->Close;
+		exit 1;
+	    };
+	    last;
+	};
+	$SIG{__WARN__} = sub {
+	    
+	    if (open DEB, ">>$::global_logfile") {
+		print DEB timestamp." -- $_[0]";
+		close DEB;
+	    } else {
+		print STDERR timestamp." -- $_[0]" 
+	    }
+	};
+	
+	
+	$SIG{__DIE__} = sub {
+	    
+	    if ( open DEB, ">>$::global_logfile") {
+		print DEB timestamp." -- $_[0]";
+		close DEB;
+	    } else {
+		print STDERR timestamp." -- $_[0]" 
+	    }
+	    exit 1
+	};
+	
+    }
+}    
+       
 =head1 NAME
 
 MRTG_lib.pm - Library for MRTG and support scripts
@@ -1614,6 +1692,20 @@ C<datestr(time)>
 Returns the time given in the argument as a nicely formated date string.
 The argument has to be in UNIX time format (seconds since 1970-1-1).
 
+=item C<timestamp>
+
+C<timestamp()>
+
+Return a string representing the current time.
+
+=item C<setup_loghandlers>
+
+C<setup_loghandlers(filename)>
+
+Install signalhandlers for __DIE__ and __WARN__ making the errors
+go the the specified destination. If filename is 'eventlog'
+mrtg will log to the windows event logger.
+
 =item C<expistr>
 
 C<expistr(time)>
