@@ -40,7 +40,8 @@ use strict qw(vars subs);	# cannot use strict subs here
 use Exporter;
 use vars qw(@ISA $VERSION @EXPORT $errmsg $suppress_warnings);
 use Socket;
-use BER;
+use BER "0.86";
+use Carp;
 
 sub map_table ($$$ );
 sub map_table_4 ($$$$);
@@ -48,7 +49,7 @@ sub map_table_start_end ($$$$$$);
 sub index_compare ($$);
 sub oid_diff ($$);
 
-$VERSION = '0.83';
+$VERSION = '0.86';
 
 @ISA = qw(Exporter);
 
@@ -118,18 +119,18 @@ sub retries { $_[0]->{retries} }
 sub backoff { $_[0]->{backoff} }
 sub set_timeout {
     my ($session, $timeout) = @_;
-    die "timeout ($timeout) must be a positive number" unless $timeout > 0.0;
+    croak ("timeout ($timeout) must be a positive number") unless $timeout > 0.0;
     $session->{'timeout'} = $timeout;
 }
 sub set_retries {
     my ($session, $retries) = @_;
-    die "retries ($retries) must be a non-negative integer"
+    croak ("retries ($retries) must be a non-negative integer")
 	unless $retries == int ($retries) && $retries >= 0;
     $session->{'retries'} = $retries; 
 }
 sub set_backoff {
     my ($session, $backoff) = @_;
-    die "backoff ($backoff) must be a number >= 1.0"
+    croak ("backoff ($backoff) must be a number >= 1.0")
 	unless $backoff == int ($backoff) && $backoff >= 1.0;
     $session->{'backoff'} = $backoff; 
 }
@@ -348,7 +349,7 @@ sub error_return ($$) {
     $SNMP_Session::errmsg = $message;
     unless ($SNMP_Session::suppress_warnings) {
 	$message =~ s/^/  /mg;
-	warn ("Error:\n".$message."\n");
+	carp ("Error:\n".$message."\n");
     }
     return undef;
 }
@@ -360,7 +361,7 @@ sub error ($$) {
     unless ($SNMP_Session::suppress_warnings) {
 	$session =~ s/^/  /mg;
 	$message =~ s/^/  /mg;
-	warn ("SNMP Error:\n".$SNMP_Session::errmsg."\n");
+	carp ("SNMP Error:\n".$SNMP_Session::errmsg."\n");
     }
     return undef;
 }
@@ -502,7 +503,7 @@ sub open {
        $remote_hostname,$community,$port,
        $max_pdu_len,$local_port,$max_repetitions,
        $local_hostname) = @_;
-    my($remote_addr,$local_addr,$socket);
+    my($remote_addr,$socket);
 
     $community = 'public' unless defined $community;
     $port = SNMP_Session::standard_udp_port unless defined $port;
@@ -513,16 +514,12 @@ sub open {
 	$remote_addr = inet_aton ($remote_hostname)
 	    or return $this->error_return ("can't resolve \"$remote_hostname\" to IP address");
     }
-    if (defined $local_hostname) {
-	$local_addr = inet_aton ($local_hostname)
-	    or return $this->error_return ("can't resolve \"$local_hostname\" to IP address");
-    }
     if ($SNMP_Session::recycle_socket && defined $the_socket) {
 	$socket = $the_socket;
     } else {
 	$socket = IO::Socket::INET->new(Proto => 17,
 					Type => SOCK_DGRAM,
-					LocalAddr => $local_addr,
+					LocalAddr => $local_hostname,
 					LocalPort => $local_port)
 	    || return $this->error_return ("creating socket: $!");
 	$the_socket = $socket
@@ -547,8 +544,8 @@ sub open {
 	   'error_index' => 0,
 	   'default_max_repetitions' => $max_repetitions,
 	   'use_getbulk' => 1,
-           'only_ip_address_matching' => 1,
 	   'lenient_source_address_matching' => 1,
+	   'lenient_source_port_matching' => 0,
 	  };
 }
 
@@ -649,15 +646,12 @@ sub sa_equal_p ($$$) {
     my ($this, $sa1, $sa2) = @_;
     my ($p1, $a1) = sockaddr_in ($sa1);
     my ($p2, $a2) = sockaddr_in ($sa2);
-
-    if ( $this->{'only_ip_address_matching'}) {
-        return 1 if $a1 eq $a2;
-    }
-
     if (! $this->{'lenient_source_address_matching'}) {
 	return 0 if $a1 ne $a2;
     }
-    return 0 if $p1 != $p2;
+    if (! $this->{'lenient_source_port_matching'}) {
+	return 0 if $p1 != $p2;
+    }
     return 1;
 }
 
@@ -679,8 +673,8 @@ sub receive_response_3 {
     if (defined $this->{'remote_addr'}) {
 	if (! $this->sa_equal_p ($remote_addr, $this->{'remote_addr'})) {
 	    if ($this->{'debug'} && !$SNMP_Session::recycle_socket) {
-		warn "Response came from ".&SNMP_Session::pretty_address($remote_addr)
-		    .", not ".&SNMP_Session::pretty_address($this->{'remote_addr'})
+		carp ("Response came from ".&SNMP_Session::pretty_address($remote_addr)
+		      .", not ".&SNMP_Session::pretty_address($this->{'remote_addr'}))
 			unless $SNMP_Session::suppress_warnings;
 	    }
 	    return 0;
@@ -693,10 +687,10 @@ sub receive_response_3 {
     if ($response_community ne $this->{community}
         || $response_id ne $this->{request_id}) {
 	if ($this->{'debug'}) {
-	    warn "$response_community != $this->{community}"
+	    carp ("$response_community != $this->{community}")
 		unless $SNMP_Session::suppress_warnings
 		    || $response_community eq $this->{community};
-	    warn "$response_id != $this->{request_id}"
+	    carp ("$response_id != $this->{request_id}")
 		unless $SNMP_Session::suppress_warnings
 		    || $response_id == $this->{request_id};
 	}
